local addonInfo, privateVars = ...

if not nkQuestBase then nkQuestBase = {} else return end

nkQuestBase = {
	updateDate = '26.4.2015',
	query = {}
}

privateVars.db = { quests = {} }
privateVars.init = false
privateVars.lang = Inspect.System.Language()
privateVars.internal = {}
privateVars.package = {}

function nkQuestBase.loadPackage(packageName)
  
  for idx = 1, #privateVars.package, 1 do
    if privateVars.package[idx] == packageName then
      return
    end
  end
  
  table.insert(privateVars.package, packageName)

end

function privateVars.internal.loader()

  if privateVars.itemLoader ~= nil and privateVars.itemLoader['all'] ~= nil then
   privateVars.itemLoader['all']()
   privateVars.itemLoader['all'] = nil
  end
    
  if #privateVars.package == 0 then return end

	if privateVars.forceUpdate ~= true then
		if privateVars.lastUpdate == nil then
      	privateVars.lastUpdate = Inspect.Time.Real()
      	privateVars.forceUpdate = true
    	else
      	local tmpTime = Inspect.Time.Real()
      	if EnKai.tools.math.round((tmpTime - privateVars.lastUpdate), 1) > .5 then privateVars.forceUpdate = true end
    	end
  	end
  
  	if privateVars.forceUpdate == true then
  		
  	local thisPackage = privateVars.package[1]
  	
  	if privateVars.questLoader[thisPackage] ~= nil then
  		
  		privateVars.questLoader[thisPackage]()
  		privateVars.questLoader[thisPackage] = nil
  		
  		privateVars.zoneLoader[thisPackage]()
      privateVars.zoneLoader[thisPackage] = nil
      
      privateVars.npcLoader[thisPackage]()
      privateVars.npcLoader[thisPackage] = nil
      
      privateVars.npcquestLoader[thisPackage]()
      privateVars.npcquestLoader[thisPackage] = nil
      
      table.remove(privateVars.package, 1)
    else  
      EnKai.tools.error.display ("nkQuestDbase", "Cannot find questLoader for package " .. thisPackage, 1)
    end
		
    privateVars.lastUpdate = Inspect.Time.Real()
    privateVars.forceUpdate = false
	end
	
	if #privateVars.package == 0 then
		Command.Event.Detach(Event.System.Update.Begin, nil, "nkQuestBase.System.Update.Begin")
		
		privateVars.questLoader = nil
		privateVars.zoneLoader = nil
		privateVars.itemLoader = nil
		privateVars.npcLoader = nil
		
		privateVars.init = true	
	end

end

function privateVars.internal.loadQuests (questTable)

	for k, v in pairs(questTable) do
		local lvl = v.lvl
		v.lvl = nil
		if privateVars.db.quests[lvl] == nil then privateVars.db.quests[lvl] = {} end
		privateVars.db.quests[lvl][k] = v
	end

end

function privateVars.internal.loadZones (zoneTable)

  if privateVars.db.byZone == nil then
    privateVars.db.byZone = {}
  end

  for k, v in pairs(zoneTable) do
    if privateVars.db.byZone[k] == nil then
      privateVars.db.byZone[k] = v;
    else
      for idx = 1, #v, 1 do
        table.insert(privateVars.db.byZone[k], v[idx])
      end
    end
  end

end

function privateVars.internal.loadNPCs (npcTable)

  if privateVars.db.npc == nil then
    privateVars.db.npc = {}
  end

  for k, v in pairs(npcTable) do
    privateVars.db.npc[k] = v;
  end

end

function privateVars.internal.loadItems (itemTable)

  if privateVars.db.questItems == nil then
    privateVars.db.questItems = {}
  end

  for k, v in pairs(itemTable) do
    privateVars.db.questItems[k] = v;
  end

end

function privateVars.internal.loadNPCQuests (npcTable)

  if privateVars.db.npcQuests == nil then
    privateVars.db.npcQuests = {}
  end

  for k, v in pairs(npcTable) do
    privateVars.db.npcQuests[k] = v
  end

end

Command.Event.Attach(Event.System.Update.Begin, privateVars.internal.loader, "nkQuestBase.System.Update.Begin")

function nkQuestBase.query.isInit() return privateVars.init end

function nkQuestBase.query.getTypeText (queryType)

	local typeList = {"Kill", "Event Objective","Daily", "Collect", "Breadcrumb", "Usable", "World Event", "Talk", "Carnage", "Use Item", "Rift", "Boss Fight", "Deliver",
							"Use Ability", "Discover", "Area Quest", "Zone Event", "Instance", "Destroy", "Escort", "Mayhem Zone Event", "Defend", "Subdue", "Quest", "Work Order"};

	if type(queryType) ~= 'number' then return "UNKNOWN_TYPE" end

	return typeList[queryType]

end

function nkQuestBase.query.getZoneByQuest (questID)
	
	for zoneID, zoneQuests in pairs(privateVars.db.byZone) do
		for _, key in pairs (zoneQuests) do
			if key == questID then return zoneID end
		end
	end

	return "UKNOWN_ZONE"

end

function nkQuestBase.query.IsQuestInZone (zoneID, questID)

	for k, v in pairs(privateVars.db.byZone[zoneID]) do
		if v == questID then return true end
	end
	
	return false

end

function nkQuestBase.query.getQuestsByZone(zoneID)

	return privateVars.db.byZone[zoneID]

end

function nkQuestBase.query.byKey (questID, zoneFlag)

	for lvl, list in pairs(privateVars.db.quests) do
		if list[questID] ~= nil then
			if zoneFlag == true and list[questID].zoneId == nil then
				list[questID].zoneId = nkQuestBase.query.getZoneByQuest (questID)
			end
			return lvl, list[questID] 
		end
	end
	
	return nil, nil

end

function nkQuestBase.query.questItemByKey (itemKey)

	return privateVars.db.questItems[itemKey]

end

function nkQuestBase.query.NPC (unitID)

	return privateVars.db.npc[unitID]

end

function nkQuestBase.query.NPCByName (checkName)

  local lang = EnKai.tools.lang.getLanguageShort()
  
  local retList = {}
  
  for id, details in pairs(privateVars.db.npc) do
    if details[lang] == checkName then table.insert(retList, id) end
  end
  
  if #retList > 0 then
    return retList
  else
    return nil
  end
  
end

function nkQuestBase.query.NPCQuests(unitID)

  return privateVars.db.npcQuests[unitID]

end